#include <cmath>
#include <string>
#include <fstream>
#include <iostream>

#include <spaceTimeTest.hpp>

#undef DOLOTSOFSIZES
namespace SRCH=searchTime;

namespace
{

double LogBaseTwo( const double& x )
{
  return( std::log(x) / std::log(2) );
}

}

int main( void )
{
  const std::uint32_t nSearches( 16384 );
  const std::uint32_t minNumItemsInContainers( 128 );
  const std::uint32_t maxNumItemsInContainers( 8192 );
  const std::uint32_t stepNumItemsInContainers( 512 );

  std::uint32_t spaceInEach( 0 );
  std::uint32_t totalVectorTimeThisSize( 0 );
  std::uint32_t totalMapTimeThisSize( 0 );
  std::uint32_t totalNumSearches( 0 );

#ifdef DOLOTSOFSIZES
  for( std::uint32_t nItemsInContainers = minNumItemsInContainers;
       nItemsInContainers < maxNumItemsInContainers;
       nItemsInContainers += stepNumItemsInContainers )
#else
  const std::uint32_t nItemsInContainers( maxNumItemsInContainers );
#endif
  {
    SRCH::spaceTimeTest tst( nSearches, nItemsInContainers );
    spaceInEach = tst.GetAmountOfSpaceInEachContainer();
    tst.doTimeVectorTest();
    tst.doTimeMapTest();

#ifdef DOLOTSOFSIZES
    const double lgNumItems( LogBaseTwo(nItemsInContainers) );
#else
    const double lgNumItems( 1 );
#endif
    totalVectorTimeThisSize += static_cast<std::uint32_t>(
      (static_cast<double>(tst.GetVectorTimeBinary()) / lgNumItems) );
    totalMapTimeThisSize += static_cast<std::uint32_t>(
      (static_cast<double>(tst.GetMapTime()) / lgNumItems) );
    totalNumSearches += nSearches;
  }

  std::cout
    << spaceInEach << ','
    << totalVectorTimeThisSize << ','
    << totalMapTimeThisSize
    << std::endl;

  return( 0 );
}
