#include <spaceTimeTest.hpp>

#include <pthread.h>
#include <sys/resource.h>

#include <ctime>
#include <algorithm>
#include <functional>

#include <boost/thread/thread.hpp>
#include <boost/random/mersenne_twister.hpp>
#include <boost/random/uniform_int_distribution.hpp>

namespace
{
  boost::random::mt19937 searchForGen( std::time( 0 ) );
}

namespace SRCH=searchTime;

SRCH::spaceTimeTest::spaceTimeTest(
  const std::uint32_t numSearches,
  const std::uint32_t numItemsInContainers ) :
  m_numSearches( numSearches ),
  m_numItemsInContainers( numItemsInContainers )
{
  this->populateContainers();
}

SRCH::spaceTimeTest::~spaceTimeTest()
{
}

void SRCH::spaceTimeTest::doTimeVectorTest( void )
{
  boost::thread sThrd(
    std::bind(
      &SRCH::spaceTimeTest::internal_doTimeVectorTest,
      this ) );
  sThrd.join();
}

void SRCH::spaceTimeTest::doTimeMapTest( void )
{
  boost::thread sThrd(
    std::bind(
      &SRCH::spaceTimeTest::internal_doTimeMapTest,
      this ) );
  sThrd.join();
}

const std::uint32_t
SRCH::spaceTimeTest::GetMapTime( void ) const
{
  return( (this->m_mapUsage.tv_sec * 1e6) + this->m_mapUsage.tv_usec );
}

const std::uint32_t
SRCH::spaceTimeTest::GetVectorTimeBinary( void ) const
{
  return( (this->m_vectUsage.tv_sec * 1e6) + this->m_vectUsage.tv_usec );
}

std::uint32_t SRCH::spaceTimeTest::GetAmountOfSpaceInEachContainer( void ) const
{
  return( this->m_vector.front().second.GetAmountOfSpace() );
}

void SRCH::spaceTimeTest::populateContainers( void )
{
  boost::random::uniform_int_distribution<> uniDist( 0, this->m_numItemsInContainers );
  this->m_searchKeys.reserve( this->m_numSearches );
  for( std::uint32_t sNum = 0; sNum < this->m_numSearches; ++sNum )
    this->m_searchKeys.push_back( uniDist( searchForGen ) );

  this->m_vector.reserve( this->m_numItemsInContainers );
  for( std::uint32_t i = 0; i < this->m_numItemsInContainers; ++i )
  {
    keyConsumerType curConsPair(
      std::move( std::make_pair( i, std::move( SRCH::spaceConsumer() ) ) ) );
    this->m_map.insert( curConsPair );
    this->m_vector.push_back( curConsPair );
  }
  std::sort(
    this->m_vector.begin(),
    this->m_vector.end(),
    []( const vectorType::value_type& v1, const vectorType::value_type& v2 ) -> bool
  {
    return( v1.first < v2.first );
  } );
}

void SRCH::spaceTimeTest::internal_doTimeVectorTest( void )
{
  keyConsumerType toSearchFor( std::move( std::make_pair(
					    this->m_searchKeys.front(),
					    spaceConsumer() ) ) );
  for( std::uint32_t sNum = 0; sNum < this->m_numSearches; ++sNum )
  {
    toSearchFor.first = this->m_searchKeys.at( sNum );
    std::binary_search(
      m_vector.cbegin(),
      m_vector.cend(),
      toSearchFor,
      []( const vectorType::value_type& v1, const vectorType::value_type& v2 ) -> bool
      {
	return( v1.first == v2.first );
      } );
  }
  struct rusage curUsage;
  getrusage( RUSAGE_THREAD, &curUsage );
  this->m_vectUsage = curUsage.ru_utime;
}

void SRCH::spaceTimeTest::internal_doTimeMapTest( void )
{
  for( std::uint32_t sNum = 0; sNum < this->m_numSearches; ++sNum )
  {
    const std::uint32_t toSearchFor( this->m_searchKeys.at( sNum ) );
    this->m_map.find( toSearchFor );
  }
  struct rusage curUsage;
  getrusage( RUSAGE_THREAD, &curUsage );
  this->m_mapUsage = curUsage.ru_utime;
}
