#include <map>
#include <vector>
#include <cstdint>
#include <utility>
#include <boost/utility.hpp>
#include <sys/time.h>

#include <spaceConsumer.hpp>

namespace searchTime
{

class spaceTimeTest : private boost::noncopyable
{
public:

  spaceTimeTest(
    const std::uint32_t numSearches,
    const std::uint32_t numItemsInContainers );
  ~spaceTimeTest();

  void doTimeVectorTest( void );
  void doTimeMapTest( void );

  const std::uint32_t GetMapTime( void ) const;
  const std::uint32_t GetVectorTimeBinary( void ) const;

  std::uint32_t GetAmountOfSpaceInEachContainer( void ) const;

private:

  typedef std::pair<std::uint32_t, spaceConsumer> keyConsumerType;
  typedef std::map<std::uint32_t, spaceConsumer> mapType;
  typedef std::vector<keyConsumerType> vectorType;
  typedef std::vector<std::uint32_t> searchKeyContainerType;

  const std::uint32_t m_numSearches;
  const std::uint32_t m_numItemsInContainers;
  mapType m_map;
  vectorType m_vector;
  searchKeyContainerType m_searchKeys;
  struct timeval m_vectUsage;
  struct timeval m_mapUsage;

  spaceTimeTest();
  void populateContainers( void );
  void internal_doTimeVectorTest( void );
  void internal_doTimeMapTest( void );
};

}
